@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2015 Red Hat, Inc.
+// Copyright (C) 2017 Red Hat, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
 //
 // Authors: Daniel Kopecek <dkopecek@redhat.com>
 //          Jiri Vymazal   <jvymazal@redhat.com>
+//          Radovan Sroka <rsroka@redhat.com>
 //
 #ifdef HAVE_BUILD_CONFIG_H
   #include <build-config.h>
@@ -25,7 +26,9 @@
 #include "FileAuditBackend.hpp"
 #include "LinuxAuditBackend.hpp"
 #include "Common/Utility.hpp"
+#include "NSHandler.hpp"
 
+#include "usbguard/DeviceManager.hpp"
 #include "usbguard/Logger.hpp"
 #include "usbguard/RuleParser.hpp"
 #include "usbguard/Audit.hpp"
@@ -61,11 +64,13 @@ namespace usbguard
     "IPCAllowedGroups",
     "DeviceRulesWithPort",
     "InsertedDevicePolicy",
+    "AuthorizedDefault",
     "RestoreControllerDeviceState",
     "DeviceManagerBackend",
     "IPCAccessControlFiles",
     "AuditFilePath",
-    "AuditBackend"
+    "AuditBackend",
+    "HidePII"
   };
 
   static const std::vector<std::pair<std::string, Daemon::DevicePolicyMethod>> device_policy_method_strings = {
@@ -100,7 +105,6 @@ namespace usbguard
 
   Daemon::Daemon()
     : _config(G_config_known_names),
-      _ruleset(this),
       _audit(_audit_identity)
   {
     sigset_t signal_set;
@@ -144,8 +148,12 @@ namespace usbguard
         if (  !( file_stat.st_mode & permissions ) ||
           ( file_stat.st_mode & permission_bad)
         ) {
-          USBGUARD_LOG(Error) << "Policy leaks are possible with this permissions!";
-          throw Exception("Check permissions", path, "Policy leaks possible");
+          std::ostringstream strm ;
+          strm.width(4) ;
+          strm.fill('0') ;
+          strm << std::oct << permissions ;
+          USBGUARD_LOG(Error) << "Permissions for " << path << " should be " <<  strm.str() ;
+          throw Exception("Check permissions", path, "Policy may be readable");
         }
         else {
           USBGUARD_LOG(Info) << "File has correct permissions.";
@@ -153,7 +161,7 @@ namespace usbguard
       }
       else {
         USBGUARD_LOG(Error) << "ERROR: File is not a regular file.";
-        throw Exception("Check permissions", path, "File not exists");
+        throw Exception("Check permissions", path, "Path is not a file");
       }
     }
     else {
@@ -173,27 +181,21 @@ namespace usbguard
     }
 
     _config.open(path, /*readonly=*/true);
+    USBGUARD_LOG(Info) << "Loading NSSwitch...";
+    _nss.parseNSSwitch();
 
     /* RuleFile */
     if (_config.hasSettingValue("RuleFile")) {
-      const std::string& rule_file = _config.getSettingValue("RuleFile");
+      const std::string& rulefile_path = _config.getSettingValue("RuleFile");
 
-      try {
-        loadRules(rule_file, check_permissions);
-      }
-      catch (const RuleParserError& ex) {
-        throw Exception("Configuration", rule_file, ex.hint());
-      }
-      catch (const std::exception& ex) {
-        throw Exception("Configuration", rule_file, ex.what());
-      }
-      catch (...) {
-        throw Exception("Configuration", rule_file, "unknown exception");
+      if (check_permissions) {
+        checkPermissions(rulefile_path, (S_IRUSR | S_IWUSR));
       }
+
+      _nss.setRulesPath(rulefile_path);
     }
-    else {
-      USBGUARD_LOG(Warning) << "RuleFile not set; Modification of the permanent policy won't be possible.";
-    }
+
+    loadRules();
 
     /* ImplicitPolicyTarget */
     if (_config.hasSettingValue("ImplicitPolicyTarget")) {
@@ -270,6 +272,13 @@ namespace usbguard
 
     _dm = DeviceManager::create(*this, _device_manager_backend);
 
+    /* RestoreControllerDeviceState */
+    if (_config.hasSettingValue("AuthorizedDefault")) {
+      const std::string value = _config.getSettingValue("AuthorizedDefault");
+      DeviceManager::AuthorizedDefaultType authorized_default = DeviceManager::authorizedDefaultTypeFromString(value);
+      _dm->setAuthorizedDefault(authorized_default);
+    }
+
     /* RestoreControllerDeviceState */
     if (_config.hasSettingValue("RestoreControllerDeviceState")) {
       const std::string value = _config.getSettingValue("RestoreControllerDeviceState");
@@ -340,18 +349,40 @@ namespace usbguard
       }
     }
 
+    /* HidePII */
+    if (_config.hasSettingValue("HidePII")) {
+      const std::string value = _config.getSettingValue("HidePII");
+
+      if (value == "true") {
+        _audit.setHidePII(true);
+      }
+      else if (value != "false") {
+        throw Exception("Configuration", "HidePII", "Invalid value");
+      }
+    }
+
     USBGUARD_LOG(Info) << "Configuration loaded successfully.";
   }
 
-  void Daemon::loadRules(const std::string& path, const bool check_permissions)
+  void Daemon::loadRules()
   {
-    USBGUARD_LOG(Info) << "Loading permanent policy file " << path;
+    USBGUARD_LOG(Info) << "Loading RuleSet";
+    auto ruleset = _nss.getRuleSet(this);
 
-    if (check_permissions) {
-      checkPermissions(path, (S_IRUSR | S_IWUSR));
+    try {
+      ruleset->load();
+    }
+    catch (const RuleParserError& ex) {
+      throw Exception("Rules", _nss.getSourceInfo(), ex.hint());
+    }
+    catch (const std::exception& ex) {
+      throw Exception("Rules", _nss.getSourceInfo(), ex.what());
+    }
+    catch (...) {
+      throw Exception("Rules", _nss.getSourceInfo(), "unknown exception");
     }
 
-    _ruleset.load(path);
+    _policy.setRuleSet(ruleset);
   }
 
   void Daemon::loadIPCAccessControlFiles(const std::string& path)
@@ -425,7 +456,7 @@ namespace usbguard
   {
     USBGUARD_LOG(Debug) << "Setting ImplicitPolicyTarget to " << Rule::targetToString(target);
     _implicit_policy_target = target;
-    _ruleset.setDefaultTarget(target);
+    _policy.setDefaultTarget(target);
   }
 
   void Daemon::setPresentDevicePolicyMethod(DevicePolicyMethod policy)
@@ -589,7 +620,7 @@ namespace usbguard
 
   uint32_t Daemon::assignID()
   {
-    return _ruleset.assignID();
+    return _policy.assignID();
   }
 
   /*
@@ -610,10 +641,10 @@ namespace usbguard
       << " parent_insensitive=" << parent_insensitive;
     const Rule match_rule = Rule::fromString(match_spec);
     const Rule new_rule = Rule::fromString(rule_spec);
-    const uint32_t id = _ruleset.upsertRule(match_rule, new_rule, parent_insensitive);
+    const uint32_t id = _policy.upsertRule(match_rule, new_rule, parent_insensitive);
 
     if (_config.hasSettingValue("RuleFile")) {
-      _ruleset.save(_config.getSettingValue("RuleFile"));
+      _policy.getRuleSet()->save();
     }
 
     USBGUARD_LOG(Trace) << "return: id=" << id;
@@ -628,10 +659,22 @@ namespace usbguard
     if (name == "InsertedDevicePolicy") {
       const auto previous_value = devicePolicyMethodToString(_inserted_device_policy_method);
       setInsertedDevicePolicyMethod(devicePolicyMethodFromString(value));
+      PropertyParameterChanged(name,
+        previous_value,
+        value);
       return previous_value;
     }
-
-    throw Exception("setParameter", name, "unknown parameter");
+    else if (name == "ImplicitPolicyTarget") {
+      const auto previous_value = Rule::targetToString(_implicit_policy_target);
+      setImplicitPolicyTarget(Rule::targetFromString(value));
+      PropertyParameterChanged(name,
+        previous_value,
+        value);
+      return previous_value;
+    }
+    else {
+      throw Exception("setParameter", name, "unknown parameter");
+    }
   }
 
   std::string Daemon::getParameter(const std::string& name)
@@ -639,22 +682,26 @@ namespace usbguard
     if (name == "InsertedDevicePolicy") {
       return devicePolicyMethodToString(_inserted_device_policy_method);
     }
-
-    throw Exception("getParameter", name, "unknown parameter");
+    else if (name == "ImplicitPolicyTarget") {
+      return Rule::targetToString(_implicit_policy_target);
+    }
+    else {
+      throw Exception("getParameter", name, "unknown parameter");
+    }
   }
 
   uint32_t Daemon::appendRule(const std::string& rule_spec,
-    uint32_t parent_id)
+    uint32_t parent_id, bool permanent)
   {
     USBGUARD_LOG(Trace) << "entry:"
       << " rule_spec=" << rule_spec
       << " parent_id=" << parent_id;
     const Rule rule = Rule::fromString(rule_spec);
     /* TODO: reevaluate the firewall rules for all active devices */
-    const uint32_t id = _ruleset.appendRule(rule, parent_id);
+    const uint32_t id = _policy.getRuleSet()->appendRule(rule, parent_id);
 
-    if (_config.hasSettingValue("RuleFile")) {
-      _ruleset.save(_config.getSettingValue("RuleFile"));
+    if (_config.hasSettingValue("RuleFile") && permanent) {
+      _policy.getRuleSet()->save();
     }
 
     USBGUARD_LOG(Trace) << "return: id=" << id;
@@ -664,17 +711,26 @@ namespace usbguard
   void Daemon::removeRule(uint32_t id)
   {
     USBGUARD_LOG(Trace) << "id=" << id;
-    _ruleset.removeRule(id);
+    _policy.removeRule(id);
 
     if (_config.hasSettingValue("RuleFile")) {
-      _ruleset.save(_config.getSettingValue("RuleFile"));
+      _policy.getRuleSet()->save();
     }
   }
 
-  const RuleSet Daemon::listRules(const std::string& query)
+  const std::vector<Rule> Daemon::listRules(const std::string& label)
   {
-    USBGUARD_LOG(Trace) << "entry: query=" << query;
-    return _ruleset;
+    USBGUARD_LOG(Trace) << "entry: label=" << label;
+    std::vector<Rule> rules;
+
+    for(auto const& rule : _policy.getRuleSet()->getRules()) {
+      if (label.empty() || rule->getLabel() == label) {
+        rules.push_back(*rule);
+      }
+    }
+
+    USBGUARD_LOG(Trace) << "return:" << " count(rules)=" << rules.size();
+    return rules;
   }
 
   uint32_t Daemon::applyDevicePolicy(uint32_t id, Rule::Target target, bool permanent)
@@ -797,7 +853,7 @@ namespace usbguard
       break;
 
     case DevicePolicyMethod::ApplyPolicy:
-      policy_rule = _ruleset.getFirstMatchingRule(device_rule);
+      policy_rule = _policy.getFirstMatchingRule(device_rule);
       break;
 
     case DevicePolicyMethod::Allow:
@@ -847,7 +903,7 @@ namespace usbguard
       break;
 
     case DevicePolicyMethod::ApplyPolicy:
-      matched_rule = _ruleset.getFirstMatchingRule(device_rule);
+      matched_rule = _policy.getFirstMatchingRule(device_rule);
       target = matched_rule->getTarget();
       break;
 
@@ -927,6 +983,7 @@ namespace usbguard
       break;
 
     case Rule::Target::Invalid:
+    case Rule::Target::Empty:
     case Rule::Target::Unknown:
     case Rule::Target::Match:
     case Rule::Target::Device:
@@ -945,7 +1002,7 @@ namespace usbguard
     USBGUARD_LOG(Debug) << "rule_spec=" << rule_spec;
     /* Upsert */
     const uint32_t rule_id = upsertRule(match_spec, rule_spec, /*parent_insensitive=*/true);
-    auto upsert_rule = _ruleset.getRule(rule_id);
+    auto upsert_rule = _policy.getRule(rule_id);
     USBGUARD_LOG(Trace) << "return:"
       << " upsert_rule=" << upsert_rule->toString();
     return upsert_rule;
