@@ -48,6 +48,29 @@
 
 namespace usbguard
 {
+  namespace
+  {
+    void setDeviceAuthorizedDefault(SysFSDevice* device, DeviceManager::AuthorizedDefaultType auth_default)
+    {
+      if (auth_default == DeviceManager::AuthorizedDefaultType::Keep) {
+        return;
+      }
+
+      std::string auth_default_str = std::to_string(DeviceManager::authorizedDefaultTypeToInteger(auth_default));
+      device->setAttribute("authorized_default", auth_default_str);
+
+      if (device->readAttribute("authorized_default", /*strip_last_null=*/true) != auth_default_str) {
+        if (auth_default == DeviceManager::AuthorizedDefaultType::Internal) {
+          USBGUARD_LOG(Warning) << "No kernel support for authorized_default = 2, falling back to 0";
+          setDeviceAuthorizedDefault(device, DeviceManager::AuthorizedDefaultType::None);
+        }
+        else {
+          throw Exception("UEventDevice", device->getPath(), "Failed to set authorized_default to \"" + auth_default_str + "\"");
+        }
+      }
+    }
+  }  /* namespace */
+
   UEventDevice::UEventDevice(UEventDeviceManager& device_manager, SysFSDevice& sysfs_device)
     : Device(device_manager)
   {
@@ -103,6 +126,10 @@ namespace usbguard
       setTarget(Rule::Target::Block);
     }
 
+    /*
+     * Set connect type
+     */
+    setConnectType(sysfs_device.readAttribute("port/connect_type", /*strip_last_null=*/true, /*optional=*/true));
     /*
      * Process USB descriptor data.
      *
@@ -213,7 +240,7 @@ namespace usbguard
     }
 
     const USBDeviceDescriptor* const device_descriptor = \
-      reinterpret_cast<const USBDeviceDescriptor* const>(descriptor);
+      reinterpret_cast<const USBDeviceDescriptor*>(descriptor);
 
     if (device_descriptor->idVendor == 0x1d6b /* Linux Foundation */) {
       switch (device_descriptor->idProduct) {
@@ -233,7 +260,7 @@ namespace usbguard
   void UEventDevice::updateHashLinuxRootHubDeviceDescriptor(const USBDescriptor* const descriptor)
   {
     USBGUARD_LOG(Trace);
-    USBDeviceDescriptor descriptor_modified = *reinterpret_cast<const USBDeviceDescriptor* const>(descriptor);
+    USBDeviceDescriptor descriptor_modified = *reinterpret_cast<const USBDeviceDescriptor*>(descriptor);
     descriptor_modified.bcdDevice = 0;
     updateHash(&descriptor_modified, sizeof descriptor_modified);
   }
@@ -248,17 +275,13 @@ namespace usbguard
       _wakeup_fd(-1),
       _enumeration(false)
   {
-    setDefaultBlockedState(/*state=*/true);
     setEnumerationOnlyMode(/*state=*/false);
-    USBGUARD_SYSCALL_THROW("UEvent device manager",
-      (_wakeup_fd = eventfd(0, 0)) < 0);
-    _uevent_fd = ueventOpen();
   }
 
   UEventDeviceManager::~UEventDeviceManager()
   {
     if (getRestoreControllerDeviceState()) {
-      setDefaultBlockedState(/*state=*/false); // FIXME: Set to previous state
+      setAuthorizedDefault(AuthorizedDefaultType::All); // FIXME: Set to previous state
     }
 
     stop();
@@ -272,11 +295,6 @@ namespace usbguard
     }
   }
 
-  void UEventDeviceManager::setDefaultBlockedState(bool state)
-  {
-    _default_blocked_state = state;
-  }
-
   void UEventDeviceManager::setEnumerationOnlyMode(bool state)
   {
     _enumeration_only_mode = state;
@@ -284,19 +302,27 @@ namespace usbguard
 
   void UEventDeviceManager::start()
   {
+    // Lazy initialization is used for the sockets to allow scanning a devpath
+    // without needed to open the socket or start the thread.
+    USBGUARD_SYSCALL_THROW("UEvent device manager",
+      (_wakeup_fd = eventfd(0, 0)) < 0);
+    _uevent_fd = ueventOpen();
     _thread.start();
   }
 
   void UEventDeviceManager::stop()
   {
-    // stop monitor
-    _thread.stop(/*do_wait=*/false);
-    { /* Wakeup the device manager thread */
-      const uint64_t one = 1;
-      USBGUARD_SYSCALL_THROW("Linux device manager",
-        write(_wakeup_fd, &one, sizeof one) != sizeof one);
+    if (_thread.running()) {
+      // stop monitor
+      _thread.stop(/*do_wait=*/false);
+      { /* Wakeup the device manager thread */
+        const uint64_t one = 1;
+        USBGUARD_SYSCALL_THROW("Linux device manager",
+          write(_wakeup_fd, &one, sizeof one)
+          != sizeof one);
+      }
+      _thread.wait();
     }
-    _thread.wait();
   }
 
   void UEventDeviceManager::scan()
@@ -316,6 +342,27 @@ namespace usbguard
     }
   }
 
+  void UEventDeviceManager::scan(const std::string& devpath)
+  {
+    std::vector<std::string> components;
+    tokenizeString(devpath, components, "/", /*trim_empty=*/true);
+    auto base = std::find_if(components.begin(), components.end(),
+    [](const std::string& component) -> bool {
+      return hasPrefix(component, "usb");
+    });
+    std::string path = "";
+
+    for (auto itr = components.begin(); itr < components.end(); ++itr) {
+      path += "/" + *itr;
+
+      if (itr >= base) {
+        ueventProcessAction("add", path);
+      }
+    }
+
+    ueventProcessAction("add", path);
+  }
+
   std::shared_ptr<Device> UEventDeviceManager::applyDevicePolicy(uint32_t id, Rule::Target target)
   {
     USBGUARD_LOG(Trace) << "id=" << id
@@ -324,7 +371,7 @@ namespace usbguard
     std::unique_lock<std::mutex> device_lock(device->refDeviceMutex());
     sysfsApplyTarget(device->sysfsDevice(), target);
     device->setTarget(target);
-    return std::move(device);
+    return device;
   }
 
   int UEventDeviceManager::ueventOpen()
@@ -383,6 +430,7 @@ namespace usbguard
     case Rule::Target::Match:
     case Rule::Target::Device:
     case Rule::Target::Unknown:
+    case Rule::Target::Empty:
     case Rule::Target::Invalid:
     default:
       throw std::runtime_error("Unknown rule target in applyDevicePolicy");
@@ -496,7 +544,7 @@ namespace usbguard
     }
 
     const struct ucred* const cmsg_ucred = \
-        reinterpret_cast<const struct ucred* const>(CMSG_DATA(cmsg_header));
+        reinterpret_cast<const struct ucred*>(CMSG_DATA(cmsg_header));
 
     if (cmsg_ucred == nullptr) {
       /* missing ucred -- ignore */
@@ -546,6 +594,11 @@ namespace usbguard
     }
 
     const std::string sysfs_devpath = uevent.getAttribute("DEVPATH");
+    ueventProcessAction(action, sysfs_devpath);
+  }
+
+  void UEventDeviceManager::ueventProcessAction(const std::string& action, const std::string& sysfs_devpath)
+  {
     bool enumeration_notify = false;
 
     try {
@@ -801,10 +854,12 @@ namespace usbguard
   {
     try {
       std::shared_ptr<UEventDevice> device = std::make_shared<UEventDevice>(*this, sysfs_device);
+      DeviceManager::AuthorizedDefaultType auth_default = getAuthorizedDefault();
 
       if (device->isController() && !_enumeration_only_mode) {
-        USBGUARD_LOG(Debug) << "Setting default blocked state for controller device to " << _default_blocked_state;
-        device->sysfsDevice().setAttribute("authorized_default", _default_blocked_state ? "0" : "1");
+        USBGUARD_LOG(Debug) << "Setting default blocked state for controller device to " <<
+          DeviceManager::authorizedDefaultTypeToString(auth_default);
+        setDeviceAuthorizedDefault(&device->sysfsDevice(), auth_default);
       }
 
       insertDevice(device);
