@@ -70,12 +70,12 @@ namespace usbguard
     UEventDeviceManager(DeviceManagerHooks& hooks);
     ~UEventDeviceManager();
 
-    void setDefaultBlockedState(bool state) override;
     void setEnumerationOnlyMode(bool state) override;
 
     void start() override;
     void stop() override;
     void scan() override;
+    void scan(const std::string& devpath) override;
 
     std::shared_ptr<Device> applyDevicePolicy(uint32_t id, Rule::Target target) override;
     void insertDevice(std::shared_ptr<UEventDevice> device);
@@ -83,6 +83,7 @@ namespace usbguard
 
     uint32_t getIDFromSysfsPath(const std::string& syspath) const;
 
+
   private:
     static bool ueventEnumerateComparePath(const std::pair<std::string, std::string>& a,
       const std::pair<std::string, std::string>& b);
@@ -94,6 +95,7 @@ namespace usbguard
     int ueventOpen();
     void ueventProcessRead();
     void ueventProcessUEvent(const UEvent& uevent);
+    void ueventProcessAction(const std::string& action, const std::string& sysfs_devpath);
     int ueventEnumerateDevices();
     int ueventEnumerateTriggerDevice(const std::string& devpath, const std::string& buspath);
 
@@ -113,7 +115,6 @@ namespace usbguard
 
     std::map<std::string, uint32_t> _sysfs_path_to_id_map;
 
-    bool _default_blocked_state;
     bool _enumeration_only_mode;
     std::atomic<bool> _enumeration;
     std::condition_variable _enumeration_complete;
